unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComObj, ActiveX, AutoCAD_TLB, ExtCtrls;

type
  TForm1 = class(TForm)
    GroupBox2: TGroupBox;
    btnPrompt: TButton;
    btnGetPoint: TButton;
    btnGetCorner: TButton;
    btnGetAngle: TButton;
    btnGetDistance: TButton;
    btnGetInteger: TButton;
    btnGetEntity: TButton;
    GroupBox3: TGroupBox;
    btnSelectOnScreen: TButton;
    SelectOnScreenAll: TButton;
    btnSelectAtPoint: TButton;
    GroupBox1: TGroupBox;
    rgrpSelectByPolygonMode: TRadioGroup;
    btnSelectByPolygon: TButton;
    Label1: TLabel;
    Image1: TImage;
    procedure FormCreate(Sender: TObject);
    procedure btnSelectOnScreenClick(Sender: TObject);
    procedure SelectOnScreenAllClick(Sender: TObject);
    procedure btnSelectAtPointClick(Sender: TObject);
    procedure btnSelectByPolygonClick(Sender: TObject);
    procedure btnGetPointClick(Sender: TObject);
    procedure btnGetCornerClick(Sender: TObject);
    procedure btnGetAngleClick(Sender: TObject);
    procedure btnGetDistanceClick(Sender: TObject);
    procedure btnGetIntegerClick(Sender: TObject);
    procedure btnPromptClick(Sender: TObject);
    procedure btnGetEntityClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

uses UAutoCAD, Math;

{$R *.dfm}

const
  SelSetName = 'TestSelSet';

procedure TForm1.FormCreate(Sender: TObject);
begin
  Acad:= GetActiveOleObject('AutoCAD.Application');
end;

procedure TForm1.btnSelectOnScreenClick(Sender: TObject);
var
  SelSet, FilterType, FilterData: OleVariant;
begin
  try
    //  SelSet
    SelSet:= Acad.ActiveDocument.SelectionSets.Add(SelSetName);
  except
    SelSet:= Acad.ActiveDocument.SelectionSets.Item(SelSetName);
    SelSet.Delete;
    SelSet:= Acad.ActiveDocument.SelectionSets.Add(SelSetName);
  end;
  FilterType:= VarArrayCreate([0, 3], varSmallint);
  FilterData:= VarArrayCreate([0, 3], varVariant);
  //    Selection Set    
  FilterType[0]:= -4;
  FilterData[0]:= '<or';
  FilterType[1]:= 0;
  FilterData[1]:= 'Arc';
  FilterType[2]:= 0;
  FilterData[2]:= 'Circle';
  FilterType[3]:= -4;
  FilterData[3]:= 'or>';
  //      
  SelSet.SelectOnScreen(FilterType, FilterData);
end;

procedure TForm1.SelectOnScreenAllClick(Sender: TObject);
var
  SelSet, Receding: OleVariant;
begin
  try
    //  SelSet
    SelSet:= Acad.ActiveDocument.SelectionSets.Add(SelSetName);
  except
    SelSet:= Acad.ActiveDocument.SelectionSets.Item(SelSetName);
    SelSet.Delete;
    SelSet:= Acad.ActiveDocument.SelectionSets.Add(SelSetName);
  end;
  //      
  SelSet.SelectOnScreen();
  ShowMessage(Format(' %d (-)', [Integer(SelSet.Count)]));
  //     
  Receding:= VarArrayCreate([0, 0], varDispatch);
  Receding[0]:= SelSet.Item(0);
  SelSet.RemoveItems(Receding);
  ShowMessage(Format('   %d (-)', [Integer(SelSet.Count)]));
end;

procedure TForm1.btnSelectAtPointClick(Sender: TObject);
var
  SelSet: OleVariant;
begin
  try
    SelSet:= Acad.ActiveDocument.SelectionSets.Add(SelSetName);
  except
    SelSet:= Acad.ActiveDocument.SelectionSets.Item(SelSetName);
    SelSet.Delete;
    SelSet:= Acad.ActiveDocument.SelectionSets.Add(SelSetName);
  end;
  //    ,    (50, 50, 0)
  SelSet.SelectAtPoint(AcadPoint(50, 50));
  // ...   
  if MessageBox(Handle, PChar(Format('  %s?', [SelSet.Item(0).ObjectName])),
                        PChar(Application.Title), MB_YESNO) = IDYES then SelSet.Erase;
end;

procedure TForm1.btnSelectByPolygonClick(Sender: TObject);
var
  SelSet, PointsList: OleVariant;
  Mode: AcSelect;
begin
  try
    SelSet:= Acad.ActiveDocument.SelectionSets.Add(SelSetName);
  except
    SelSet:= Acad.ActiveDocument.SelectionSets.Item(SelSetName);
    SelSet.Delete;
    SelSet:= Acad.ActiveDocument.SelectionSets.Add(SelSetName);
  end;
  //   
  PointsList:= VarArrayCreate([0, 11], varDouble);
  PointsList[0]:= 0; PointsList[1]:= 0; PointsList[2]:= 0;
  PointsList[3]:= 100; PointsList[4]:= 0; PointsList[5]:= 0;
  PointsList[6]:= 100; PointsList[7]:= 100; PointsList[8]:= 0;
  PointsList[9]:= 0; PointsList[10]:= 100; PointsList[11]:= 0;
  case rgrpSelectByPolygonMode.ItemIndex of
    0: Mode:= acSelectionSetFence;
    1: Mode:= acSelectionSetWindowPolygon;
    2: Mode:= acSelectionSetCrossingPolygon;
  end;
  SelSet.SelectByPolygon(Mode, PointsList);
  //  
  if MessageBox(Handle, PChar(Format(' %d ()?', [Integer(SelSet.Count)])),
                        PChar(Application.Title), MB_YESNO) = IDYES then SelSet.Erase;
end;

procedure TForm1.btnGetPointClick(Sender: TObject);
var
  Pnt1, Pnt2: OleVariant;
begin
  //      
  Pnt1:= Acad.ActiveDocument.Utility.GetPoint(, '  : ');
  //         " "   
  Pnt2:= Acad.ActiveDocument.Utility.GetPoint(Pnt1, '  : ');
  //       
  Acad.ActiveDocument.ModelSpace.AddLine(Pnt1, Pnt2);
end;

procedure TForm1.btnGetCornerClick(Sender: TObject);
var
  Pnt1, Pnt2, Pnt3, Pnt4: OleVariant;
begin
  //      
  Pnt1:= Acad.ActiveDocument.Utility.GetPoint(, '  : ');
  //         " "   
  Pnt2:= Acad.ActiveDocument.Utility.GetCorner(Pnt1, '  : ');
  //    
  Pnt3:= AcadPoint(Pnt1[0], Pnt2[1]);
  Pnt4:= AcadPoint(Pnt2[0], Pnt1[1]);
  //  
  Acad.ActiveDocument.ModelSpace.AddLine(Pnt1, Pnt3);
  Acad.ActiveDocument.ModelSpace.AddLine(Pnt3, Pnt2);
  Acad.ActiveDocument.ModelSpace.AddLine(Pnt2, Pnt4);
  Acad.ActiveDocument.ModelSpace.AddLine(Pnt4, Pnt1);
end;

procedure TForm1.btnGetAngleClick(Sender: TObject);
var
  Ang: Double;
begin
  //    
  Ang:= Acad.ActiveDocument.Utility.GetAngle(, '  : ');
  ShowMessage(Format('  %f ', [RadToDeg(Ang)]));
end;

procedure TForm1.btnGetDistanceClick(Sender: TObject);
var
  Dist: Double;
begin
  //       
  Dist:= Acad.ActiveDocument.Utility.GetDistance(, 'Select first point: ');
  //      
  Acad.ActiveDocument.Utility.Prompt(Format('Distance = %f', [Dist]));
end;

procedure TForm1.btnGetIntegerClick(Sender: TObject);
var
  Int: Smallint;
begin
  //     
  Int:= Acad.ActiveDocument.Utility.GetInteger('   <-32768...32767>: ');
  //     
  Acad.ActiveDocument.Utility.Prompt(Format(' = %d', [Int]));
end;

procedure TForm1.btnPromptClick(Sender: TObject);
begin
  Acad.ActiveDocument.Utility.Prompt('Press any key...');
  Acad.Update;
end;

procedure TForm1.btnGetEntityClick(Sender: TObject);
var
  BasePnt, Obj: OleVariant;
  Entity: IDispatch;
  RetStr: String;
  OldColor: TOleEnum;
  Flag: Boolean;
begin
  Flag:= True;
  while Flag do // ,    N(n)  Esc
  begin
    try
      Acad.ActiveDocument.Utility.GetEntity(Entity, BasePnt, '  ');
      Obj:= Entity;
      //    
      OldColor:= Obj.Color;
      Obj.Color:= acRed;
      Obj.Update;
      Acad.ActiveDocument.Utility.InitializeUserInput(128, 'Yes No');
      RetStr:= Acad.ActiveDocument.Utility.GetKeyword(Format(' : %s. ? (<Yes>/No): ', [Obj.EntityName]));
      if RetStr = 'No' then Flag:= False;
      Obj.Color:= OldColor;
      Obj.Update;
    except
       Flag:= False;
    end;
  end;
end;

end.
